// ----------------------------------------------------------------------
// Phone Service MAIN View Model
// ----------------------------------------------------------------------
function PhoneFunkyViewModel() {
	var self = this;

	self.log = function(msg) {
		if (typeof console != "undefined")
			console.log(msg);
	};

	self.load_page = function(page_file) {
		self.log("load_page: " + page_file);

		self.show_header(true);
		self.show_options(false);
		self.back_page("");
		self.refresh_function(null);
		self.add_function(null);
		self.del_function(null);

		if (self.analytics != null) {
			var page_title = page_file.substring(0, page_file.indexOf("."));
			self.log("Track Analytics: " + page_title);
			self.analytics.trackView(page_title);
		}

		jQuery('#content').load(page_file, null, function() {
		});
	};

	self.show_header = ko.observable(true);
	self.show_options = ko.observable(false);
	self.back_page = ko.observable("");
	self.refresh_function = ko.observable(null);
	self.add_function = ko.observable(null);
	self.del_function = ko.observable(null);

	self.back = function() {
		self.load_page(self.back_page());
	};

	self.refresh = function() {
		self.refresh_function()();
	};

	self.add = function() {
		self.add_function()();
	};
	
	self.del = function() {
		self.del_function()();
	}

	self.device_type = function() {
		if ((typeof device != "undefined") && (device != null))
			return device.platform;
		return "WEB";
	};

	self.timezonename = null;
	self.timezone = function() {
		if (!self.timezonename) {
			var tz = jstz.determine();
			self.timezonename = tz.name();
			self.log("Timezone: " + self.timezonename);
		}
		return self.timezonename;
	};

	self.sigInViewModel = new PhoneFunkySigInViewModel(self);
	self.logInViewModel = new PhoneFunkyLogInViewModel(self);
	self.passwordResetViewModel = new PhoneFunkyPasswordResetViewModel(self);
	self.alertsViewModel = new PhoneFunkyAlertsViewModel(self);
	self.optionsViewModel = new PhoneFunkyOptionsViewModel(self);
	self.passwordChangeViewModel = new PhoneFunkyPasswordChangeViewModel(self);
	self.deleteAccountViewModel = new PhoneFunkyDeleteAccountViewModel(self);
	self.servicesViewModel = new PhoneFunkyServicesViewModel(self);
	self.numbersViewModel = new PhoneFunkyNumbersViewModel(self);
	self.addNumberViewModel = new PhoneFunkyAddNumberViewModel(self);
	self.servicesChiMiHaChiamatoViewModel = new ServicesChiMiHaChiamatoViewModel(self);
	self.contactsViewModel = new PhoneFunkyContactsViewModel(self);

	self.contactManager = new ContactManager(self);

	self.rpc = function(pars) {
		var data_to_log = [];
		data_to_log.params = pars;

		var url = "http://www.phonefunky.com/wp-admin/admin-ajax.php";

		if (window.location.href.indexOf("local.") >= 0)
			url = "http://local.phonefunky.com:8080/wp-admin/admin-ajax.php";

		if (typeof pars.data == "undefined" || pars.data == null)
			pars.data = [];
		if (typeof pars.success == "undefined")
			pars.success = null;
		if (typeof pars.error == "undefined")
			pars.error = null;
		if (typeof pars.always == "undefined")
			pars.always = null;
		if (typeof pars.autologout == "undefined")
			pars.autologout = true;

		pars.data["device_type"] = self.device_type();

		self.log("rpc");
		self.log(pars);

		jQuery.ajax({
			url : url,
			type : 'post',
			data : pars.data,
			success : function(data_str, status, xhr) {
				self.log("rpc success: " + data_str);
				// data = jQuery.parseJSON(data_str);
				data = (data_str);
				data_to_log.response = data;
				self.log(data_to_log);
				if (data["result"] == "NOT_LOGGED" && pars.autologout) {
					self.load_page("login.html");
				} else if (pars.success)
					pars.success(data);
				if (pars.always)
					pars.always(true, data);
			},
			error : function(errorThrown) {
				self.log("rpc error: " + errorThrown);
				self.log(JSON.stringify(errorThrown));
				data_to_log.errorThrow = errorThrown;
				self.log(data_to_log);
				if (pars.error)
					pars.error(errorThrown);
				if (pars.always)
					pars.always(false, errorThrown);
			}
		});
	};

	// ----------------------------------------------------------------------
	// CORDOVA SPECIFIC
	// ----------------------------------------------------------------------
	// Ad Mob
	self.admob_testing = false;
	self.admob_ios_key = 'ca-app-pub-3388912755637151/8655866224';
	self.admob_android_key = 'ca-app-pub-3388912755637151/1112579823';
	self.adId = "";

	self.admob_ios_key_interstitial = 'ca-app-pub-3388912755637151/1132599428';
	self.admob_android_key_interstitial = 'ca-app-pub-3388912755637151/4066046221';
	self.adId_interstitial = "";

	self.initAdMob = function() {
		self.adId = (navigator.userAgent.indexOf('Android') >= 0) ? self.admob_android_key : self.admob_ios_key;
		self.adId_interstitial = (navigator.userAgent.indexOf('Android') >= 0) ? self.admob_android_key_interstitial : self.admob_ios_key_interstitial;
	};

	self.showAdMobBanner = function() {
        if(AdMob) 
            AdMob.createBanner({
                adId: self.adId,
                position: AdMob.AD_POSITION.TOP_CENTER,
                autoShow: true });		
	        };

	self.hideAdMobBanner = function() {
        if(AdMob)	
            AdMob.removeBanner();	
	};
    
    self.showAdMobInterstitial = function() {        
        if(AdMob) {
            var next_show = localStorage.getItem("ad_interstitial_next_show") ? localStorage.getItem("ad_interstitial_next_show") : 0;
            var cur_time = new Date().getTime();

            if (next_show < cur_time) {
                AdMob.prepareInterstitial( {adId:self.adId_interstitial, autoShow:true} );
                next_show = cur_time + 90*1000;
                localStorage.setItem('ad_interstitial_next_show', next_show);
            } else  
                self.log("showAdMobInterstitial - Wait " + (next_show - cur_time)/1000 + " secs");                 
        }
    }

	// Push
	self.pushNotification = null;
	self.pushNotificationToken = "";
	self.appVersion = "";

	self.analytics = null;

	self.onDeviceReady = function() {
		self.log("OnDeviceReady");
		
		self.contactManager.update();
		
		self.pushNotificationToken = localStorage.getItem("pushNotificationToken") ? localStorage.getItem("pushNotificationToken") : "";

        // AppVersion
		cordova.getAppVersion.getVersionNumber(function (version) {
			self.log("AppVersion: " + version);
			self.appVersion = version;
		});
		
		// Register GA
		if ((typeof analytics != "undefined")) {
			self.log("Registering Analytics");
			self.analytics = analytics;
			self.analytics.startTrackerWithId('UA-34725092-6');
		}
		
        // Push
		self.pushNotification = PushNotification.init({
		    android: {
        		senderID: "642976803927"
		    },
		    ios: {
        		alert: "true",
		        badge: "true",
        		sound: "true"
		    },
		    windows: {}
		});
		
		self.pushNotification.on('registration', function(data) {
		    // data.registrationId
			self.log('pushNotificationTokenHandler => token : ' + data.registrationId);
			self.pushNotificationToken = data.registrationId;
			self.pushNotificationReLogin();		    
		});

		self.pushNotification.on('notification', function(data) {
		    // data.message,
		    // data.title,
		    // data.count,
		    // data.sound,
		    // data.image,
		    // data.additionalData
		    
			if (data.additionalData && data.additionalData.code && data.additionalData.code == "NEWCALL")
				self.load_page("alerts.html");
			if (data.additionalData && data.additionalData.code && data.additionalData.code == "OPENURL")
				window.open(e.payload.url, '_system', 'location=yes');
		});

		self.pushNotification.on('error', function(e) {
		    // e.message
		    self.log('pushNotificationErrorHandler => ' + e.message);
		});		

		// Init AdMob
		self.initAdMob();

		// self.showAdMobBanner();
	};
	
	self.pushNotificationReLogin = function() {
		self.log("pushNotificationReLogin");

		if ((self.logInViewModel.username() != "") && (self.logInViewModel.password() != "")) {
			self.rpc({
				data : {
					action : "phonefunky_login",
					username : self.logInViewModel.username(),
					password : self.logInViewModel.password(),
					device : self.device_type(),
					pushtoken : self.pushNotificationToken,
					appversion : self.appVersion
				},
				autologout : false,
				success : function(data) {
				},
				error : function(error) {
				},
				always : function(result, dataOrError) {
				}
			});
		}

	};
}

// ----------------------------------------------------------------------
// Contacts Manager
// ----------------------------------------------------------------------
function ContactManager(owner) {
	var self = this;
	self.owner = owner;

	self.contacts = [];

	self.get_name_for_number = function(number) {
		var result = "";

		jQuery.each(self.contacts, function(idx, itm) {
			if (itm.number.indexOf(number) >= 0) {
				result = itm.name;
				return false;
			} else if (number.length >= 8 && itm.number.indexOf(number.substring(1)) >= 0) {
				result = itm.name;
				return false;
			} else if (number.length >= 9 && itm.number.indexOf(number.substring(2)) >= 0) {
				result = itm.name;
				return false;
			} else if (number.length >= 10 && itm.number.indexOf(number.substring(3)) >= 0) {
				result = itm.name;
				return false;
			}
		});

		return result;
	};

    self.ready = false;
	self.update = function() {
		if (typeof navigator.contacts != "undefined") {
			// self.owner.log("LOAD CONTACTS");

			function onSuccess(contacts) {
				if (contacts && (contacts.length > 0)) {
					self.contacts = [];
					self.owner.log("ContactManager found: " + contacts.length);

					jQuery.each(contacts, function(idx, val) {
						var name = val["name"]["formatted"];
						// name = val["displayName"];

						if (val["phoneNumbers"])
							jQuery.each(val["phoneNumbers"], function(idx2, val2) {
								var number = val2["value"];
								number = number.replace(/\s+/g, '');

								self.contacts.push({
									number : number,
									name : name
								});
								// self.owner.log("FOUND: " + name + " -> " + number);
							});

					});
                    
                    self.ready = true;
				}
			}
			function onError(contactError) {
			}

			var options = new ContactFindOptions();
			options.filter = "";
			options.multiple = true;
			var fields = [ navigator.contacts.fieldType.name ];
			navigator.contacts.find(fields, onSuccess, onError, options);
		}
	}

}

// ----------------------------------------------------------------------
// Phone Services SIGIN View Model
// ----------------------------------------------------------------------
function PhoneFunkySigInViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.user_email = ko.observable("");

	self.working = ko.observable(false);
	self.working_invalid_email = ko.observable(false);
	self.working_email_exists = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_registered = ko.observable(false);

	self.init = function() {
		self.user_email("");

		self.working(false);
		self.working_invalid_email(false);
		self.working_email_exists(false);
		self.working_error(false);
		self.working_registered(false);

		self.owner.show_options(false);
		self.owner.back_page("");
	};

	self.register = function() {
		self.working(true);
		self.working_invalid_email(false);
		self.working_email_exists(false);
		self.working_error(false);
		self.working_registered(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_sigin",
				email : self.user_email()
			},
			success : function(data) {
				if (data["result"] == "OK")
					self.working_registered(true);
				else if (data["result"] == "INVALID_EMAIL")
					self.working_invalid_email(true);
				else if (data["result"] == "EMAIL_ALREADY_EXISTS")
					self.working_email_exists(true);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services LOGIN View Model
// ----------------------------------------------------------------------
function PhoneFunkyLogInViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.logged = ko.observable(false);
	self.username = ko.observable("");
	self.password = ko.observable("");
	self.remember_me = ko.observable(false);

	self.working = ko.observable(false);
	self.working_invalid_account = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_loggedin = ko.observable(false);

	self.load_from_storage = function() {
		if (typeof (Storage) !== "undefined") {
			if (localStorage.remember_me) {
				self.username((localStorage.username ? localStorage.username : ""));
				self.password((localStorage.password ? localStorage.password : ""));
				self.remember_me(true);
			}
		}
	};

	self.save_to_storage = function() {
		if (self.remember_me()) {
			if (typeof (Storage) !== "undefined") {
				localStorage.setItem("username", self.username());
				localStorage.setItem("password", self.password());
				localStorage.setItem("remember_me", true);
			}
		} else {
			if (typeof (Storage) !== "undefined") {
				localStorage.setItem("username", "");
				localStorage.setItem("password", "");
				localStorage.setItem("remember_me", false);
			}
		}
	};

	self.init = function() {
		self.logged(false);
		self.username("");
		self.password("");

		self.load_from_storage();

		self.working(false);
		self.working_invalid_account(false);
		self.working_error(false);
		self.working_loggedin(false);

		self.owner.show_header(false);
		self.owner.show_options(false);
		self.owner.back_page("");

		//if (typeof self.owner.hideAdMobBanner != "undefined")
		//	self.owner.hideAdMobBanner();
	};

	self.login = function() {
		self.logged(false);

		self.working(true);
		self.working_invalid_account(false);
		self.working_error(false);
		self.working_loggedin(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_login",
				username : self.username(),
				password : self.password(),
				device : self.owner.device_type(),
				pushtoken : self.owner.pushNotificationToken,
				appversion : self.owner.appVersion
			},
			success : function(data) {
				if (data["result"] == "OK") {
					self.logged(true);
					self.save_to_storage();
					// if (self.owner.device_type() != "WEB" &&
					// self.owner.pushNotificationToken == "")
					// self.working_error(true);
					// else {
					self.working_loggedin(true);
					if (typeof self.owner.showAdMobBanner != "undefined")
						self.owner.showAdMobBanner();
					owner.load_page("alerts.html");
					// }
				} else if (data["result"] == "INVALID_ACCOUNT")
					self.working_invalid_account(true);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});

	};

	self.go_to_site = function() {
		window.open("http://www.phonefunky.com", '_system', 'location=yes');
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services LOGIN View Model
// ----------------------------------------------------------------------
function PhoneFunkyPasswordResetViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.username = ko.observable("");

	self.working = ko.observable(false);
	self.working_invalid_email = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_resetted = ko.observable(false);

	self.init = function() {
		self.username("");

		self.working(false);
		self.working_invalid_email(false);
		self.working_error(false);
		self.working_resetted(false);

		self.owner.show_options(false);
		self.owner.back_page("");
	};

	self.reset = function() {
		self.working(true);
		self.working_invalid_email(false);
		self.working_error(false);
		self.working_resetted(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_password_reset",
				username : self.username()

			},
			success : function(data) {
				if (data["result"] == "OK" || data["result"] == "INVALID_ACCOUNT")
					self.working_resetted(true);
				else if (data["result"] == "INVALID_EMAIL")
					self.working_invalid_email(true);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});

	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services ALERTS View Model
// ----------------------------------------------------------------------
function PhoneFunkyAlertsViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.alerts = ko.observableArray();
	self.numbers = ko.observableArray();

	self.working = ko.observable(false);
	self.working_error = ko.observable(false);

	self.init = function() {
		self.owner.show_options(true);
		self.owner.back_page("");
		self.owner.refresh_function(self.refresh);
		self.owner.del_function(null);
	};

    self.refresh_retry = 0;
    
	self.refresh = function() {
		self.working(true);
		self.working_error(false);

		// self.owner.showAdMobInterstitial();

		self.alerts([]);
		
        if ((!self.owner.contactManager.ready) && (self.refresh_retry<10)) {
            self.refresh_retry++;
            self.owner.log("ContactManager NOT ready... wating... " + self.refresh_retry);
            setTimeout(function() {
                self.refresh();
            }, 1000);
            return;
        }
        
		self.owner.rpc({
			data : {
				action : "phonefunky_get_alerts",
				timezone : self.owner.timezone()
			},
			success : function(data) {
				if (data["result"] == "OK") {
					jQuery.each(data["alerts"], function(idx, val) {
						data["alerts"][idx]["alert"] = jQuery.parseJSON(data["alerts"][idx]["alert"]);
						if (data["alerts"][idx]["type"] == "NEWCALL") {
							data["alerts"][idx]["alert"]["contact"] = ko.observable(self.owner.contactManager.get_name_for_number(data["alerts"][idx]["alert"]["caller_number"]));
						}
					});

					self.owner.del_function(null);
					if (data["alerts"].length>0)
						self.owner.del_function(self.remove_all);

					self.alerts(data["alerts"]);
					self.numbers(data["numbers"]);
					self.build_internal_alerts();
				} else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
				self.owner.showAdMobInterstitial();
			}
		});
	};

	self.call = function( item ) {
		if ( (item.alert.caller_number.length>=7) && item.alert.caller_number.match(/^[0-9]+$/) )
			window.open('tel:+'+item.alert.caller_number, '_system');
	}
	
	self.build_internal_alerts = function() {
		jQuery.each(self.numbers(), function(idx, itm) {
			var count_service_active = 0;

			jQuery.each(itm, function(idx2, itm2) {
				if (idx2.indexOf("services_") == 0 && itm2 == 1)
					count_service_active++;
			});

			if ((count_service_active > 0) && (itm.stats_callsdiverted == 0)) {
				var alert = {
					id : "NODIVERSION",
					internal : 1,
					date : 0,
					date_in_timezone : 0,
					type : 'NODIVERSION',
					alert : {
						prefix : itm.prefix,
						number : itm.number
					}
				};
				self.alerts.unshift(alert);
			}

			if (count_service_active == 0) {
				var alert = {
					id : "NOSERVICE",
					internal : 1,
					date : 0,
					date_in_timezone : 0,
					type : 'NOSERVICE',
					alert : {
						prefix : itm.prefix,
						number : itm.number
					}
				};
				self.alerts.unshift(alert);
			}
		});
	};

	self.remove_all = function() {		
		jQuery("#modal_deleteallcall").modal('show');			
	};
	
	self.remove = function(id) {
		self.working(true);
		self.working_error(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_delete_alert",
				id : id
			},
			success : function(data) {
				if (data["result"] == "OK") {
					self.refresh();
				} else {
					self.working_error(true);
				}
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				// self.working(false);
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services OPTIONS View Model
// ----------------------------------------------------------------------
function PhoneFunkyOptionsViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.init = function() {
		self.owner.show_options(false);
		self.owner.back_page("alerts.html");
	};

	self.logout = function() {
		self.owner.rpc({
			data : {
				action : "phonefunky_logout"
			},
			success : function(data) {
			},
			error : function(error) {
			},
			always : function(result, dataOrError) {
			}
		});

		self.owner.load_page("login.html");
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services OPTIONS View Model
// ----------------------------------------------------------------------
function PhoneFunkyPasswordChangeViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.new_password = ko.observable("");
	self.new_password_retyped = ko.observable("");

	self.working = ko.observable(false);
	self.working_password_not_match = ko.observable(false);
	self.working_invalid_password = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_password_changed = ko.observable(false);

	self.init = function() {
		self.new_password("");
		self.new_password_retyped("");

		self.working(false);
		self.working_password_not_match(false);
		self.working_invalid_password(false);
		self.working_error(false);
		self.working_password_changed(false);

		self.owner.show_options(false);
		self.owner.back_page("options.html");
	};

	self.change = function() {
		self.working(true);
		self.working_password_not_match(false);
		self.working_invalid_password(false);
		self.working_error(false);
		self.working_password_changed(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_password_change",
				new_password : self.new_password(),
				new_password_retyped : self.new_password_retyped()
			},
			success : function(data) {
				if (data["result"] == "OK") {
					self.working_password_changed(true);
				} else if (data["result"] == "PASSWORD_NOT_MATCH") {
					self.working_password_not_match(true);
				} else if (data["result"] == "INVALID_PASSWORD") {
					self.working_invalid_password(true);
				} else {
					self.working_error(true);
				}
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services DELETE_ACCOUNT View Model
// ----------------------------------------------------------------------
function PhoneFunkyDeleteAccountViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.working = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_account_deleted = ko.observable(false);

	self.init = function() {
		self.working(false);
		self.working_error(false);
		self.working_account_deleted(false);

		self.owner.show_options(false);
		self.owner.back_page("options.html");
	};

	self.confirm_delete = function() {
		self.working(true);
		self.working_error(false);
		self.working_account_deleted(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_delete_account"
			},
			success : function(data) {
				if (data["result"] == "OK") {
					self.owner.back_page("login.html");
					self.working_account_deleted(true);
				} else {
					self.working_error(true);
				}
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services NUMBER View Model
// ----------------------------------------------------------------------
function PhoneFunkyNumbersViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.working = ko.observable(false);
	self.working_error = ko.observable(false);

	self.numbers = ko.observableArray();

	self.init = function() {
		self.owner.show_options(false);
		self.owner.back_page("options.html");
		self.owner.refresh_function(self.refresh);
		self.owner.add_function(function() {
			self.owner.addNumberViewModel.back_page("numbers.html");
			self.owner.load_page("addnumber.html");
		});
	};

	self.refresh = function() {
		self.working(true);
		self.working_error(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_get_numbers"
			},
			success : function(data) {
				if (data["result"] == "OK")
					self.numbers(data["numbers"]);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.remove = function(id) {
		self.working(true);
		self.working_error(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_delete_number",
				id : id
			},
			success : function(data) {
			},
			error : function(error) {
			},
			always : function(result, dataOrError) {
				self.refresh();
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services ADD NUMBER View Model
// ----------------------------------------------------------------------
function PhoneFunkyAddNumberViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.back_page = ko.observable("numbers.html");

	self.prefix = ko.observable("");
	self.number = ko.observable("");

	self.working = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_invalid_number = ko.observable(false);
	self.working_count_down = ko.observable(false);

	self.init = function() {
		self.prefix("");
		self.number("");

		self.working(false);
		self.working_error(false);
		self.working_invalid_number(false);
		self.working_count_down(null);

		self.owner.show_options(false);
		self.owner.back_page(self.back_page());
		self.owner.refresh_function(null);
		self.owner.add_function(null);
	};

	self.add = function() {
		self.working(true);
		self.working_error(false);
		self.working_invalid_number(false);
		self.working_count_down(null);

		self.owner.rpc({
			data : {
				action : "phonefunky_add_number",
				prefix : self.prefix(),
				number : self.number()
			},
			success : function(data) {
				if (data["result"] == "OK") {
					self.working_count_down(data["timeout"]);
					self.checkNumber();
				} else if (data["result"] == "INVALID_NUMBER")
					self.working_invalid_number(true);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.checkNumber = function() {
		self.owner.rpc({
			data : {
				action : "phonefunky_check_number",
				prefix : self.prefix(),
				number : self.number()
			},
			success : function(data) {
				if (data["result"] == "OK")
					self.working_count_down(data["timeout"]);
			},
			error : function(error) {
			},
			always : function(result, dataOrError) {
				if (jQuery("#working_count_down_item").is(":visible") && self.working_count_down() > 0)
					window.setTimeout(self.checkNumber, 1000);
			}
		});			
	};
	
	self.call = function() {
		window.open('tel:+390694802015', '_system');
	};

	self.init();
}

// ----------------------------------------------------------------------
// Phone Services SERVICES View Model
// ----------------------------------------------------------------------
function PhoneFunkyServicesViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.init = function() {
		self.owner.show_options(false);
		self.owner.back_page("options.html");
	};

	self.go_to_whooming = function() {
		window.open("https://www.whooming.com", '_system');
	};

	self.go_to_sbizzy = function() {
		window.open("https://www.sbizzy.com", '_system');
	};

	self.init();
}

// ----------------------------------------------------------------------
// CONTACTS View Model
// ----------------------------------------------------------------------
function PhoneFunkyContactsViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.name = ko.observable("");
	self.email = ko.observable("");
	self.subject = ko.observable("");
	self.message = ko.observable("");

	self.working = ko.observable(false);
	self.working_sended = ko.observable(false);
	self.working_error = ko.observable(false);
	self.working_invalid_name = ko.observable(false);
	self.working_invalid_email = ko.observable(false);
	self.working_invalid_subject = ko.observable(false);
	self.working_invalid_message = ko.observable(false);

	self.init = function() {
		self.name("");
		self.email(self.owner.logInViewModel ? self.owner.logInViewModel.username() : "");
		self.subject("");
		self.message("");

		self.working(false);
		self.working_sended(false);
		self.working_error(false);

		self.working_invalid_name(false);
		self.working_invalid_email(false);
		self.working_invalid_subject(false);
		self.working_invalid_message(false);

		self.owner.show_options(false);
		self.owner.back_page("options.html");
	};

	self.send = function() {
		self.working(true);
		self.working_sended(false);
		self.working_error(false);

		self.working_invalid_name(false);
		self.working_invalid_email(false);
		self.working_invalid_subject(false);
		self.working_invalid_message(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_open_ticket",
				name : self.name(),
				email : self.email(),
				subject : self.subject(),
				message : self.message()
			},
			success : function(data) {
				if (data["result"] == "OK")
					self.working_sended(true);
				else if (data["result"] == "INVALID_NAME")
					self.working_invalid_name(true);
				else if (data["result"] == "INVALID_EMAIL")
					self.working_invalid_email(true);
				else if (data["result"] == "INVALID_SUBJECT")
					self.working_invalid_subject(true);
				else if (data["result"] == "INVALID_MESSAGE")
					self.working_invalid_message(true);
				else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.init();
}

// ----------------------------------------------------------------------
// SERVICES ChiMiHaChiamato View Model
// ----------------------------------------------------------------------
function ServicesChiMiHaChiamatoViewModel(owner) {
	var self = this;
	self.owner = owner;

	self.working = ko.observable(false);
	self.working_last_active_id = ko.observable(0);
	self.working_error = ko.observable(false);

	self.numbers = ko.observableArray();

	self.init = function() {
		self.working(false);
		self.working_last_active_id(0);
		self.working_error(false);

		self.owner.show_options(false);
		self.owner.back_page("services.html");
		self.owner.refresh_function(self.refresh);
		self.owner.add_function(function() {
			self.owner.addNumberViewModel.back_page("services_chimihachiamato.html");
			self.owner.load_page("addnumber.html")
		});
	};

	self.refresh = function(reset) {
		if (typeof reset == "undefined")
			reset = true;
		self.working(true);
		if (reset) {
			self.working_last_active_id(0);
		}
		self.working_error(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_get_numbers"
			},
			success : function(data) {
				if (data["result"] == "OK") {

					if (self.working_last_active_id() == 0) {
						if (self.owner.addNumberViewModel.number() != "") {
							jQuery.each(data["numbers"], function(idx, itm) {
								if (itm.number == self.owner.addNumberViewModel.number()) {
									self.working_last_active_id(itm.id);
									self.owner.addNumberViewModel.number("");
									return false;
								}
							});
						}
					}

					self.numbers(data["numbers"]);

				} else
					self.working_error(true);
			},
			error : function(error) {
				self.working_error(true);
			},
			always : function(result, dataOrError) {
				self.working(false);
			}
		});
	};

	self.set_active = function(id, status) {
		self.working(true);
		self.working_last_active_id(0);
		self.working_error(false);

		self.owner.rpc({
			data : {
				action : "phonefunky_set_service_active_status",
				id : id,
				service : "chimihachiamato",
				status : status
			},
			success : function(data) {
				if (data["result"] == "OK") {
					if (status == 1)
						self.working_last_active_id(id);
					self.refresh(false);
				} else {
					self.working_error(true);
					self.working(false);
				}
			},
			error : function(error) {
				self.working_error(true);
				self.working(false);
			},
			always : function(result, dataOrError) {
			}
		});
	};

	self.init();

}

ko.bindingHandlers.showModal = {
	init : function(element, valueAccessor, allBindings, viewModel, bindingContext) {
		var value = valueAccessor();
		var valueUnwrapped = ko.unwrap(value);
		if (value)
			jQuery(element).modal('show');
	},
	update : function(element, valueAccessor, allBindings, viewModel, bindingContext) {
	}
};

var phoneFunkyViewModel = new PhoneFunkyViewModel();
function pushNotificationOnNotification(e) {
	phoneFunkyViewModel.pushNotificationOnNotification(e);
}
function pushNotificationOnNotificationAPN(e) {
	phoneFunkyViewModel.pushNotificationOnNotificationAPN(e);
}